#include "TS24501Msgs.h"
#include "rtxsrc/rtxMemLeakCheck.h"
#include "rtxsrc/rtxCharStr.h"
#include "rtxsrc/rtxDiag.h"
#include "rtxsrc/rtxDiagBitTrace.h"
#include "rtxsrc/rtxError.h"
#include "rtxsrc/rtxFile.h"
#include "rtxsrc/rtxPrint.h"
#include "rt3gppsrc/rt3gppNasSec.h"

#define MAXMSGLEN 300

int main (int argc, char** argv)
{
   TS24501Msg_PDU pdu;
   TS24501Msg_DeregReqUETerm data;
   TS24501Msg_DeregReqUETerm* pvalue = &data;
   OSCTXT	ctxt;
   OSOCTET	msgbuf[MAXMSGLEN], *msgptr;
   const char*  filename = "message.dat";
   OSSIZE       len;
   int		i, ret;
   OSBOOL verbose = FALSE;
   OSCrypt128Key integKey = {
      0x01, 0x23, 0x45, 0x67, 0x89, 0xab, 0xcd, 0xef,
      0x98, 0x76, 0x54, 0x32, 0x10, 0xfe, 0xdc, 0xba
   };
   OSCrypt128Key encryptKey = {
      0x98, 0x76, 0x54, 0x32, 0x10, 0xfe, 0xdc, 0xba,
      0x01, 0x23, 0x45, 0x67, 0x89, 0xab, 0xcd, 0xef
   };

   DECLARE_MEMLEAK_DETECTOR;

   /* Process command line arguments */

   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else {
            printf ("usage: writer [-v]\n");
            printf ("   -v  verbose mode: print trace info\n");
            return 0;
         }
      }
   }

   /* Initialize context structure */

   ret = rtInitContext (&ctxt);
   if (0 != ret) {
      printf ("rtInitContext failed; status = %d\n", ret);
      rtxErrPrint (&ctxt);
      return ret;
   }

   ret = rtx3gppSecInitialize(&ctxt);
   if (0 != ret) {
      rtxErrPrint (&ctxt);
   }

   ret = rtx3gppAssignAlgorithmKeys (&ctxt, &integKey, &encryptKey,
      OS3GPPSecAlgorithm_AES, OS3GPPSecAlgorithm_AES);
   if ( ret == RTERR_NOTSUPP ) {
      printf ("Warning: unable to set AES algorithm; falling back to "
              "NULL algorithm.\n");
      rtxErrReset (&ctxt);
      ret = rtx3gppAssignAlgorithmKeys (&ctxt, NULL, NULL, 
         OS3GPPSecAlgorithm_NULL_ALG, OS3GPPSecAlgorithm_NULL_ALG);
   }
   if (0 != ret) {
      rtxErrPrint (&ctxt);
   }

   ctxt.p3gppSec->count = 15;
   ctxt.p3gppSec->bearerId = 0;
   ctxt.p3gppSec->direction = 0; /*uplink*/

   /* Populate data structure */

   asn1Init_TS24501Msg_PDU (&pdu);
   pdu.m.secHdrPresent = 0;
   pdu.hdrData.u.mm.secHdrType = TS24501Msg_NAS5GSecHdrType_noSec;

   asn1SetTC_TS24501Msg_PDU_obj_DeregReqUETerm5G (&ctxt, &pdu, pvalue);

   asn1Init_TS24501Msg_DeregReqUETerm (pvalue);

   pvalue->deregType.switchOff = 0;
   pvalue->deregType.reregReqd = 1;
   pvalue->deregType.accessType = 2;

   pvalue->m.causePresent = TRUE;
   pvalue->cause = 5;

   pvalue->m.t3346Present = 1;
   pvalue->t3346.value.unit = 5;
   pvalue->t3346.value.timerValue = 21;

   pvalue->m._v16ExtPresent = TRUE;
   pvalue->m.rejectedNSSAIPresent = TRUE;
   pvalue->rejectedNSSAI.length = 0; // Calculated automatically...
   pvalue->rejectedNSSAI.rejectedNSSAI.n = 1;
   pvalue->rejectedNSSAI.rejectedNSSAI.elem[0].length = 4;
   pvalue->rejectedNSSAI.rejectedNSSAI.elem[0].cause = TS24501IE_RejectedNSSAI_cause_authPending;
   pvalue->rejectedNSSAI.rejectedNSSAI.elem[0].sst = 150;
   pvalue->rejectedNSSAI.rejectedNSSAI.elem[0].m.sdPresent = 1;
   pvalue->rejectedNSSAI.rejectedNSSAI.elem[0].sd = 65000;

   pvalue->m._v17ExtPresent = TRUE;
   pvalue->m.extCAGInfoListPresent = TRUE;
   pvalue->extCAGInfoList.length = 0; // Calculated automatically...
   pvalue->extCAGInfoList.m.entriesPresent = TRUE;
   pvalue->extCAGInfoList.entries.n = 1;
   pvalue->extCAGInfoList.entries.elem = rtxMemAllocArrayZ(&ctxt, 1,
                                                    TS24501IE_ExtCAGEntry);
   pvalue->extCAGInfoList.entries.elem[0].length = 0; // Calculated automatically...
   rtxStrncpy(pvalue->extCAGInfoList.entries.elem[0].mcc_mnc.mcc, 4, "123", 3);
   rtxStrncpy(pvalue->extCAGInfoList.entries.elem[0].mcc_mnc.mnc, 4, "456", 3);
   pvalue->extCAGInfoList.entries.elem[0].caili = 1;
   pvalue->extCAGInfoList.entries.elem[0].lci = FALSE;
   pvalue->extCAGInfoList.entries.elem[0].cagOnly = TRUE;
   pvalue->extCAGInfoList.entries.elem[0].m.cagIdIndLenPresent = TRUE;
   pvalue->extCAGInfoList.entries.elem[0].m.cagIdPresent = TRUE;
   pvalue->extCAGInfoList.entries.elem[0].cagIdIndLen = 4;
   pvalue->extCAGInfoList.entries.elem[0].cagId.numocts = 4;
   pvalue->extCAGInfoList.entries.elem[0].cagId.data = (OSOCTET *)rtxMemAlloc(&ctxt, 4);
   {
      static const OSOCTET testdata[] = { 0x01, 0x04, 0x01, 0x04 };
      OSCRTLSAFEMEMCPY((void *)pvalue->extCAGInfoList.entries.elem[0].cagId.data, 4,
                       testdata, 4);
   }
   pvalue->extCAGInfoList.entries.elem[0].cagIdAddlInfoList.u.present.length = 0; // Calculated automatically...
   pvalue->extCAGInfoList.entries.elem[0].cagIdAddlInfoList.u.present.cagIdAddlInfo.n = 1;
   pvalue->extCAGInfoList.entries.elem[0].cagIdAddlInfoList.u.present.cagIdAddlInfo.elem = rtxMemAllocArrayZ(&ctxt, 1,
                                                                                              TS24501IE_CAGIdAddlInfo);
   pvalue->extCAGInfoList.entries.elem[0].cagIdAddlInfoList.u.present.cagIdAddlInfo.elem[0].length = 0; // Calculated automatically...
   pvalue->extCAGInfoList.entries.elem[0].cagIdAddlInfoList.u.present.cagIdAddlInfo.elem[0].cagId[0] = 4;
   pvalue->extCAGInfoList.entries.elem[0].cagIdAddlInfoList.u.present.cagIdAddlInfo.elem[0].cagId[1] = 6;
   pvalue->extCAGInfoList.entries.elem[0].cagIdAddlInfoList.u.present.cagIdAddlInfo.elem[0].cagId[2] = 8;
   pvalue->extCAGInfoList.entries.elem[0].cagIdAddlInfoList.u.present.cagIdAddlInfo.elem[0].cagId[3] = 4;
   pvalue->extCAGInfoList.entries.elem[0].cagIdAddlInfoList.u.present.cagIdAddlInfo.elem[0].tvii = 0;

   /* Enable bit tracing */
   if (verbose) {
      rtxDiagCtxtBitFieldListInit (&ctxt);
   }

   /* Encode data */
   rtxCtxtSetBufPtr (&ctxt, msgbuf, sizeof(msgbuf));

   ret = NASEnc_TS24501Msg_PDU (&ctxt, &pdu);
   if (0 != ret) {
      printf ("encode PDU failed; status = %d\n", ret);
      rtxErrPrint (&ctxt);
      printf ("Bit fields:\n");
      rtxDiagBitTracePrint (ctxt.pBitFldList, "");
      printf ("\n");
      rtxFreeContext (&ctxt);
      return ret;
   }
   msgptr = rtxCtxtGetMsgPtr (&ctxt);
   len = rtxCtxtGetMsgLen (&ctxt);

   if (verbose) {
      printf ("Encoding was successful\n");
      printf ("Hex dump of encoded record:\n");
      rtxHexDump (msgptr, len);
      printf ("\n");
      printf ("Bit fields:\n");
      rtxDiagBitTracePrint (ctxt.pBitFldList, "");
      printf ("\n");
   }

   /* Verify buffer is byte-aligned */
   OSRTASSERT (ctxt.buffer.bitOffset == 8 || ctxt.buffer.bitOffset == 0);

   /* Write encoded message to output file */

   ret = rtxFileWriteBinary (filename, msgptr, len);
   if (0 != ret) {
      printf ("rtxFileWriteBinary failed; status = %d\n", ret);
      rtxErrPrint (&ctxt);
      return ret;
   }

   rtxFreeContext (&ctxt);

   return 0;
}
