#include "TS24501Msgs.h"
#include "rtxsrc/rtxMemLeakCheck.h"
#include "rtxsrc/rtxCharStr.h"
#include "rtxsrc/rtxDiag.h"
#include "rtxsrc/rtxDiagBitTrace.h"
#include "rtxsrc/rtxError.h"
#include "rtxsrc/rtxFile.h"
#include "rtxsrc/rtxPrint.h"
#include "rt3gppsrc/rt3gppNasSec.h"

#define MAXMSGLEN 300

int main (int argc, char** argv)
{
   TS24501Msg_PDU pdu;
   TS24501Msg_ModifyCmd data;
   TS24501Msg_ModifyCmd* pvalue = &data;
   OSCTXT	ctxt;
   OSOCTET	msgbuf[MAXMSGLEN], *msgptr;
   const char*  filename = "message.dat";
   OSSIZE       len;
   int		i, ret;
   OSBOOL verbose = FALSE;
   OSCrypt128Key integKey = {
      0x01, 0x23, 0x45, 0x67, 0x89, 0xab, 0xcd, 0xef,
      0x98, 0x76, 0x54, 0x32, 0x10, 0xfe, 0xdc, 0xba
   };
   OSCrypt128Key encryptKey = {
      0x98, 0x76, 0x54, 0x32, 0x10, 0xfe, 0xdc, 0xba,
      0x01, 0x23, 0x45, 0x67, 0x89, 0xab, 0xcd, 0xef
   };

   DECLARE_MEMLEAK_DETECTOR;

   /* Process command line arguments */
   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else {
            printf ("usage: writer [-v]\n");
            printf ("   -v  verbose mode: print trace info\n");
            return 0;
         }
      }
   }

   /* Initialize context structure */
   ret = rtInitContext (&ctxt);
   if (0 != ret) {
      printf ("rtInitContext failed; status = %d\n", ret);
      rtxErrPrint (&ctxt);
      return ret;
   }

   ret = rtx3gppSecInitialize(&ctxt);
   if (0 != ret) {
      rtxErrPrint (&ctxt);
   }

   ret = rtx3gppAssignAlgorithmKeys (&ctxt, &integKey, &encryptKey,
      OS3GPPSecAlgorithm_AES, OS3GPPSecAlgorithm_AES);
   if ( ret == RTERR_NOTSUPP ) {
      printf ("Warning: unable to set AES algorithm; falling back to "
              "NULL algorithm.\n");
      rtxErrReset (&ctxt);
      ret = rtx3gppAssignAlgorithmKeys (&ctxt, NULL, NULL, 
         OS3GPPSecAlgorithm_NULL_ALG, OS3GPPSecAlgorithm_NULL_ALG);
   }
   if (0 != ret) {
      rtxErrPrint (&ctxt);
   }

   ctxt.p3gppSec->count = 15;
   ctxt.p3gppSec->bearerId = 0;
   ctxt.p3gppSec->direction = 0; /*uplink*/

   /* Populate data structure */
   asn1Init_TS24501Msg_PDU (&pdu);
   pdu.m.secHdrPresent = 1;
   pdu.secHdr.protoDiscr = TS24501Msg_NAS5GProtoDiscr_mobMgmt5G;
   pdu.secHdr.secHdrType = TS24501Msg_NAS5GSecHdrType_integProt;
   pdu.secHdr.msgAuthCode = 10000;
   pdu.secHdr.seqNumber  = 180;
   pdu.protoDiscr = TS24501Msg_NAS5GProtoDiscr_sessMgmt5G;
   pdu.hdrData.u.sm.pduSessIdent = TS24501Msg_PDUSessionIdentity_pduId2;
   pdu.hdrData.u.sm.procTransIdent = 145;

   asn1SetTC_TS24501Msg_PDU_obj_ModifyCmd5G (&ctxt, &pdu, pvalue);

   asn1Init_TS24501Msg_ModifyCmd (pvalue);

   pvalue->m.causePresent = 1;
   pvalue->cause = 27;

   pvalue->m.sessionAMBRPresent = 1;
   pvalue->sessionAMBR.unitAmbrDL = 16;
   pvalue->sessionAMBR.sessionAmbrDL = 233;
   pvalue->sessionAMBR.unitAmbrUL = 8;
   pvalue->sessionAMBR.sessionAmbrUL = 21;

   pvalue->m.rqTimerPresent = 1;
   pvalue->rqTimer.unit = 2;
   pvalue->rqTimer.timerValue = 19;

   pvalue->m.extProtoCfgOptsPresent = 1;
   pvalue->extProtoCfgOpts.octet4.configProtocol = 0;

   pvalue->m._v17ExtPresent = 1;
   pvalue->m.recvMBSContainerPresent = 1;
   pvalue->recvMBSContainer.length = 0; /* length will be calculated */
   pvalue->recvMBSContainer.receivedInfo.n = 1;
   pvalue->recvMBSContainer.receivedInfo.elem = rtxMemAllocArrayZ(&ctxt, 1,
                                                      TS24501IE_ReceivedMBSInfo);
   pvalue->recvMBSContainer.receivedInfo.elem[0].rejectionCause = 4;
   pvalue->recvMBSContainer.receivedInfo.elem[0].msai = TS24501IE_MSAITypes_notIncluded;
   pvalue->recvMBSContainer.receivedInfo.elem[0].md = 3;
   pvalue->recvMBSContainer.receivedInfo.elem[0].spare = 0;
   pvalue->recvMBSContainer.receivedInfo.elem[0].msci = 1;
   pvalue->recvMBSContainer.receivedInfo.elem[0].mti = TS24501IE_MTITypes_startTime;
   pvalue->recvMBSContainer.receivedInfo.elem[0].ipae = 1;
   {
      static const OSOCTET testdata[] = { 0xaa, 0xbb, 0xcc };
      OSCRTLSAFEMEMCPY((void *)pvalue->recvMBSContainer.receivedInfo.elem[0].tmgi.mbmsServiceID,
                       3, testdata, 3);
   }
   pvalue->recvMBSContainer.receivedInfo.elem[0].tmgi.m.mcc_mncPresent = 1;
   rtxStrncpy(pvalue->recvMBSContainer.receivedInfo.elem[0].tmgi.mcc_mnc.mcc, 4, "730", 3);
   rtxStrncpy(pvalue->recvMBSContainer.receivedInfo.elem[0].tmgi.mcc_mnc.mnc, 4, "321", 3);

   {
      static const OSOCTET testdata[] = { 0x54, 0x87, 0x31, 0xd8 };
      pvalue->recvMBSContainer.receivedInfo.elem[0].ipAddrInfo.u.included.srcAddrInfo.numocts = 4;
      OSCRTLSAFEMEMCPY((void *)pvalue->recvMBSContainer.receivedInfo.elem[0].ipAddrInfo.u.included.srcAddrInfo.data,
                       4, testdata, 4);
   }
   {
      static const OSOCTET testdata[] = { 0x7e, 0x1a, 0x44, 0xf9 };
      pvalue->recvMBSContainer.receivedInfo.elem[0].ipAddrInfo.u.included.destAddrInfo.numocts = 4;
      OSCRTLSAFEMEMCPY((void *)pvalue->recvMBSContainer.receivedInfo.elem[0].ipAddrInfo.u.included.destAddrInfo.data,
                       4, testdata, 4);
   }

   pvalue->recvMBSContainer.receivedInfo.elem[0].timer.u.startTime.year = 22;
   pvalue->recvMBSContainer.receivedInfo.elem[0].timer.u.startTime.month = 2;
   pvalue->recvMBSContainer.receivedInfo.elem[0].timer.u.startTime.day = 18;
   pvalue->recvMBSContainer.receivedInfo.elem[0].timer.u.startTime.hour = 14;
   pvalue->recvMBSContainer.receivedInfo.elem[0].timer.u.startTime.minute = 30;
   pvalue->recvMBSContainer.receivedInfo.elem[0].timer.u.startTime.second = 30;
   pvalue->recvMBSContainer.receivedInfo.elem[0].timer.u.startTime.timezone = 0;

   pvalue->recvMBSContainer.receivedInfo.elem[0].securityContainer.u.included.numKeySets = 1;
   pvalue->recvMBSContainer.receivedInfo.elem[0].securityContainer.u.included.keySetList.n = 1;
   OSCRTLMEMSET(pvalue->recvMBSContainer.receivedInfo.elem[0].securityContainer.u.included.keySetList.elem, 0,
                sizeof(TS24501IE_MBSSecurityKeySet));
   pvalue->recvMBSContainer.receivedInfo.elem[0].securityContainer.u.included.keySetList.elem[0].mtki = 1;
   rtxStrncpy(pvalue->recvMBSContainer.receivedInfo.elem[0].securityContainer.u.included.keySetList.elem[0].keyDomainId.mcc, 4, "410", 3);
   rtxStrncpy(pvalue->recvMBSContainer.receivedInfo.elem[0].securityContainer.u.included.keySetList.elem[0].keyDomainId.mnc, 4, "01", 3);
   {
      static const OSOCTET testdata[] = { 0x01, 0x23, 0x45, 0x67 };
      OSCRTLSAFEMEMCPY((void *)pvalue->recvMBSContainer.receivedInfo.elem[0].securityContainer.u.included.keySetList.elem[0].mskId,
                       4, testdata, 4);
   }
   {
      static const OSOCTET testdata[] = { 0x00, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff };
      OSCRTLSAFEMEMCPY((void *)pvalue->recvMBSContainer.receivedInfo.elem[0].securityContainer.u.included.keySetList.elem[0].msk,
                       16, testdata, 16);
   }
   {
      static const OSOCTET testdata[] = { 0x89, 0xab };
      OSCRTLSAFEMEMCPY((void *)pvalue->recvMBSContainer.receivedInfo.elem[0].securityContainer.u.included.keySetList.elem[0].mtkInfo.u.included.mtkId,
                       2, testdata, 2);
   }
   {
      static const OSOCTET testdata[] = { 0xcd, 0xef, 0xfe, 0xdc, 0xba, 0x98, 0x76, 0x54, 0x32, 0x10, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06 };
      OSCRTLSAFEMEMCPY((void *)pvalue->recvMBSContainer.receivedInfo.elem[0].securityContainer.u.included.keySetList.elem[0].mtkInfo.u.included.mtk,
                       16, testdata, 16);
   }

   /* Enable bit tracing */
   if (verbose) {
      rtxDiagCtxtBitFieldListInit (&ctxt);
   }

   /* Encode data */
   rtxCtxtSetBufPtr (&ctxt, msgbuf, sizeof(msgbuf));

   ret = NASEnc_TS24501Msg_PDU (&ctxt, &pdu);
   if (0 != ret) {
      printf ("encode PDU failed; status = %d\n", ret);
      rtxErrPrint (&ctxt);
      printf ("Bit fields:\n");
      rtxDiagBitTracePrint (ctxt.pBitFldList, "");
      printf ("\n");
      rtxFreeContext (&ctxt);
      return ret;
   }
   msgptr = rtxCtxtGetMsgPtr (&ctxt);
   len = rtxCtxtGetMsgLen (&ctxt);

   if (verbose) {
      printf ("Encoding was successful\n");
      printf ("Hex dump of encoded record:\n");
      rtxHexDump (msgptr, len);
      printf ("\n");
      printf ("Bit fields:\n");
      rtxDiagBitTracePrint (ctxt.pBitFldList, "");
      printf ("\n");
   }

   /* Verify buffer is byte-aligned */
   OSRTASSERT (ctxt.buffer.bitOffset == 8 || ctxt.buffer.bitOffset == 0);

   /* Write encoded message to output file */
   ret = rtxFileWriteBinary (filename, msgptr, len);
   if (0 != ret) {
      printf ("rtxFileWriteBinary failed; status = %d\n", ret);
      rtxErrPrint (&ctxt);
      return ret;
   }

   rtxFreeContext (&ctxt);

   return 0;
}
