#include "TS24501Msgs.h"
#include "rtxsrc/rtxMemLeakCheck.h"
#include "rtxsrc/rtxCharStr.h"
#include "rtxsrc/rtxDiag.h"
#include "rtxsrc/rtxDiagBitTrace.h"
#include "rtxsrc/rtxError.h"
#include "rtxsrc/rtxFile.h"
#include "rtxsrc/rtxPrint.h"
#include "rt3gppsrc/rt3gppNasSec.h"

#define MAXMSGLEN 300

int main (int argc, char** argv)
{
   TS24501Msg_PDU pdu;
   TS24501Msg_RegComplete data;
   TS24501Msg_RegComplete* pvalue = &data;
   OSCTXT	ctxt;
   OSOCTET	msgbuf[MAXMSGLEN], *msgptr;
   const char*  filename = "message.dat";
   OSSIZE       len;
   int		i, ret;
   OSBOOL verbose = FALSE;
   OSCrypt128Key integKey = {
      0x01, 0x23, 0x45, 0x67, 0x89, 0xab, 0xcd, 0xef,
      0x98, 0x76, 0x54, 0x32, 0x10, 0xfe, 0xdc, 0xba
   };
   OSCrypt128Key encryptKey = {
      0x98, 0x76, 0x54, 0x32, 0x10, 0xfe, 0xdc, 0xba,
      0x01, 0x23, 0x45, 0x67, 0x89, 0xab, 0xcd, 0xef
   };

   DECLARE_MEMLEAK_DETECTOR;

   /* Process command line arguments */

   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else {
            printf ("usage: writer [-v]\n");
            printf ("   -v  verbose mode: print trace info\n");
            return 0;
         }
      }
   }

   /* Initialize context structure */

   ret = rtInitContext (&ctxt);
   if (0 != ret) {
      printf ("rtInitContext failed; status = %d\n", ret);
      rtxErrPrint (&ctxt);
      return ret;
   }

   ret = rtx3gppSecInitialize(&ctxt);
   if (0 != ret) {
      rtxErrPrint (&ctxt);
   }

   ret = rtx3gppAssignAlgorithmKeys (&ctxt, &integKey, &encryptKey,
      OS3GPPSecAlgorithm_AES, OS3GPPSecAlgorithm_AES);
   if ( ret == RTERR_NOTSUPP ) {
      printf ("Warning: unable to set AES algorithm; falling back to "
              "NULL algorithm.\n");
      rtxErrReset (&ctxt);
      ret = rtx3gppAssignAlgorithmKeys (&ctxt, NULL, NULL, 
         OS3GPPSecAlgorithm_NULL_ALG, OS3GPPSecAlgorithm_NULL_ALG);
   }
   if (0 != ret) {
      rtxErrPrint (&ctxt);
   }

   ctxt.p3gppSec->count = 15;
   ctxt.p3gppSec->bearerId = 0;
   ctxt.p3gppSec->direction = 0; /*uplink*/

   /* Populate data structure */

   asn1Init_TS24501Msg_PDU (&pdu);
   pdu.m.secHdrPresent = 1;
   pdu.secHdr.protoDiscr = TS24501Msg_NAS5GProtoDiscr_mobMgmt5G;
   pdu.secHdr.secHdrType = TS24501Msg_NAS5GSecHdrType_integProt;
   pdu.secHdr.msgAuthCode = 3000;
   pdu.secHdr.seqNumber = 3;
   pdu.hdrData.u.mm.secHdrType = TS24501Msg_NAS5GSecHdrType_noSec;

   asn1SetTC_TS24501Msg_PDU_obj_RegComplete5G (&ctxt, &pdu, pvalue);

   asn1Init_TS24501Msg_RegComplete (pvalue);

   pvalue->m.transparentContainerPresent = 1;
   pvalue->transparentContainer.length = 0; // Calculated automatically...
#if 1
   // Test the Steering choice...
   pvalue->transparentContainer.steeringOrAck.t = T_SORTransparentContainer_steeringOrAck_steering;
   pvalue->transparentContainer.steeringOrAck.u.steering.ap = 1;
   pvalue->transparentContainer.steeringOrAck.u.steering.ack = TRUE;
   pvalue->transparentContainer.steeringOrAck.u.steering.listType = 1;
   pvalue->transparentContainer.steeringOrAck.u.steering.listInd = TRUE;
   pvalue->transparentContainer.steeringOrAck.u.steering.dataType = 0;
   {
      static const OSOCTET testdata[] = {
          0x69, 0xe2, 0x7e, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d };
      OSCRTLSAFEMEMCPY((void *)pvalue->transparentContainer.steeringOrAck.u.steering.sorMAC,
                       16, testdata, 16);
   }
   pvalue->transparentContainer.steeringOrAck.u.steering.counter[0] = 0xBA;
   pvalue->transparentContainer.steeringOrAck.u.steering.counter[1] = 0xDE;

   pvalue->transparentContainer.steeringOrAck.u.steering.m.contentPresent = TRUE;
#if 0  // Set steering.ap above to 0
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnIds.n = 2;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnIds.elem = rtxMemAllocArrayZ(&ctxt, 2,
                                                                                  TS24501IE_PLMNTechIDs);
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnIds.elem[0].id[0] = 0xaa;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnIds.elem[0].id[1] = 0xbb;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnIds.elem[0].id[2] = 0xcc;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnIds.elem[0].techId[0] = 0xdd;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnIds.elem[0].techId[1] = 0xee;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnIds.elem[1].id[0] = 0xaa;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnIds.elem[1].id[1] = 0xbb;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnIds.elem[1].id[2] = 0xcc;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnIds.elem[1].techId[0] = 0xdd;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnIds.elem[1].techId[1] = 0xee;
#else  // Set steering.ap above to 1
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.length = 10;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.plmnIds.n = 2;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.plmnIds.elem = rtxMemAllocArrayZ(&ctxt, 2,
                                                                                  TS24501IE_PLMNTechIDs);
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.plmnIds.elem[0].id[0] = 0xaa;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.plmnIds.elem[0].id[1] = 0xbb;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.plmnIds.elem[0].id[2] = 0xcc;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.plmnIds.elem[0].techId[0] = 0xdd;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.plmnIds.elem[0].techId[1] = 0xee;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.plmnIds.elem[1].id[0] = 0xaa;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.plmnIds.elem[1].id[1] = 0xbb;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.plmnIds.elem[1].id[2] = 0xcc;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.plmnIds.elem[1].techId[0] = 0xdd;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.plmnIds.elem[1].techId[1] = 0xee;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sssli = 1;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sssi = 1;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sscmi = TRUE;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.si = 1;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorCmci.u.cmcis.length = 0; // Calculated automatically...
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorCmci.u.cmcis.cmciRules.n = 1;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorCmci.u.cmcis.cmciRules.elem = rtxMemAllocArrayZ(&ctxt, 1,
                                                                                                                                   TS24501IE_CMCIRules);
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorCmci.u.cmcis.cmciRules.elem[0].length = 0; // Calculated automatically...
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorCmci.u.cmcis.cmciRules.elem[0].tsorCmTimer.unit = 4;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorCmci.u.cmcis.cmciRules.elem[0].tsorCmTimer.timerValue = 22;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorCmci.u.cmcis.cmciRules.elem[0].type = 3;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorCmci.u.cmcis.cmciRules.elem[0].value.u.sstSd.sst = 250;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorCmci.u.cmcis.cmciRules.elem[0].value.u.sstSd.sd = 20000;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorSnpnSi.u.snpnsi.length = 0; // Calculated automatically...
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorSnpnSi.u.snpnsi.clgi = 0;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorSnpnSi.u.snpnsi.clsi = 0;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorSnpnSiLs.u.snpnsils.localSnpnList.u.localSnpns.length = 0; // Calculated automatically...
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorSnpnSiLs.u.snpnsils.clglsi = 0;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorSnpnSiLs.u.snpnsils.clslsi = 1;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorSnpnSiLs.u.snpnsils.localSnpnList.u.localSnpns.snpnInfo.n = 1;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorSnpnSiLs.u.snpnsils.localSnpnList.u.localSnpns.snpnInfo.elem = rtxMemAllocArrayZ(&ctxt, 1,
                                                                                                                                                                TS24501IE_LocalizedSNPNInfo);
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorSnpnSiLs.u.snpnsils.localSnpnList.u.localSnpns.snpnInfo.elem[0].m.snpnIdPresent = TRUE;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorSnpnSiLs.u.snpnsils.localSnpnList.u.localSnpns.snpnInfo.elem[0].length = 0; // Calculated automatically...
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorSnpnSiLs.u.snpnsils.localSnpnList.u.localSnpns.snpnInfo.elem[0].laii = 1;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorSnpnSiLs.u.snpnsils.localSnpnList.u.localSnpns.snpnInfo.elem[0].timeInd = 0;
   rtxStrncpy(pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorSnpnSiLs.u.snpnsils.localSnpnList.u.localSnpns.snpnInfo.elem[0].snpnId.mcc_mnc.mcc,
              4, "620", 3);
   rtxStrncpy(pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorSnpnSiLs.u.snpnsils.localSnpnList.u.localSnpns.snpnInfo.elem[0].snpnId.mcc_mnc.mnc,
              4, "900", 3);
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorSnpnSiLs.u.snpnsils.localSnpnList.u.localSnpns.snpnInfo.elem[0].snpnId.assignMode = 15;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorSnpnSiLs.u.snpnsils.localSnpnList.u.localSnpns.snpnInfo.elem[0].snpnId.nidDigit2 = 0;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorSnpnSiLs.u.snpnsils.localSnpnList.u.localSnpns.snpnInfo.elem[0].snpnId.nidDigit1 = 1;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorSnpnSiLs.u.snpnsils.localSnpnList.u.localSnpns.snpnInfo.elem[0].snpnId.nidDigit4 = 2;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorSnpnSiLs.u.snpnsils.localSnpnList.u.localSnpns.snpnInfo.elem[0].snpnId.nidDigit3 = 3;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorSnpnSiLs.u.snpnsils.localSnpnList.u.localSnpns.snpnInfo.elem[0].snpnId.nidDigit6 = 4;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorSnpnSiLs.u.snpnsils.localSnpnList.u.localSnpns.snpnInfo.elem[0].snpnId.nidDigit5 = 5;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorSnpnSiLs.u.snpnsils.localSnpnList.u.localSnpns.snpnInfo.elem[0].snpnId.nidDigit8 = 6;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorSnpnSiLs.u.snpnsils.localSnpnList.u.localSnpns.snpnInfo.elem[0].snpnId.nidDigit7 = 7;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorSnpnSiLs.u.snpnsils.localSnpnList.u.localSnpns.snpnInfo.elem[0].snpnId.nidDigit10 = 8;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorSnpnSiLs.u.snpnsils.localSnpnList.u.localSnpns.snpnInfo.elem[0].snpnId.nidDigit9 = 9;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorSnpnSiLs.u.snpnsils.localSnpnList.u.localSnpns.snpnInfo.elem[0].locationAssistance.u.info.numDescriptions = 1;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorSnpnSiLs.u.snpnsils.localSnpnList.u.localSnpns.snpnInfo.elem[0].locationAssistance.u.info.descriptions.n = 1;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorSnpnSiLs.u.snpnsils.localSnpnList.u.localSnpns.snpnInfo.elem[0].locationAssistance.u.info.descriptions.elem =
                                                                                                                rtxMemAllocArrayZ(&ctxt, 1, TS24501IE_AreaDescriptions);
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorSnpnSiLs.u.snpnsils.localSnpnList.u.localSnpns.snpnInfo.elem[0].locationAssistance.u.info.descriptions.elem[0].shapeType = 6;
   pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorSnpnSiLs.u.snpnsils.localSnpnList.u.localSnpns.snpnInfo.elem[0].locationAssistance.u.info.descriptions.elem[0].shapeDesc.numocts = 4;
   {
      static const OSOCTET testdata[] = { 0x69, 0xe2, 0x7e, 0x01 };
      OSCRTLSAFEMEMCPY((void *)pvalue->transparentContainer.steeringOrAck.u.steering.content.u.plmnInfo.u.plmnList.addlParams.sorSnpnSiLs.u.snpnsils.localSnpnList.u.localSnpns.snpnInfo.elem[0].locationAssistance.u.info.descriptions.elem[0].shapeDesc.data,
                       4, testdata, 4);
   }
#endif
#else
   // Test the Steering ACK choice...
   pvalue->transparentContainer.steeringOrAck.t = T_SORTransparentContainer_steeringOrAck_ack;
   pvalue->transparentContainer.steeringOrAck.u.ack.mssnpnsils = FALSE;
   pvalue->transparentContainer.steeringOrAck.u.ack.mssnpnsi = TRUE;
   pvalue->transparentContainer.steeringOrAck.u.ack.mssi = TRUE;
   pvalue->transparentContainer.steeringOrAck.u.ack.dataType = 1;
   {
      static const OSOCTET testdata[] = {
          0x69, 0xe2, 0x7e, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d };
      OSCRTLSAFEMEMCPY((void *)pvalue->transparentContainer.steeringOrAck.u.ack.sorMAC,
                       16, testdata, 16);
   }
#endif

   /* Enable bit tracing */
   if (verbose) {
      rtxDiagCtxtBitFieldListInit (&ctxt);
   }

   /* Encode data */
   rtxCtxtSetBufPtr (&ctxt, msgbuf, sizeof(msgbuf));

   ret = NASEnc_TS24501Msg_PDU (&ctxt, &pdu);
   if (0 != ret) {
      printf ("encode PDU failed; status = %d\n", ret);
      rtxErrPrint (&ctxt);
      printf ("Bit fields:\n");
      rtxDiagBitTracePrint (ctxt.pBitFldList, "");
      printf ("\n");
      rtxFreeContext (&ctxt);
      return ret;
   }
   msgptr = rtxCtxtGetMsgPtr (&ctxt);
   len = rtxCtxtGetMsgLen (&ctxt);

   if (verbose) {
      printf ("Encoding was successful\n");
      printf ("Hex dump of encoded record:\n");
      rtxHexDump (msgptr, len);
      printf ("\n");
      printf ("Bit fields:\n");
      rtxDiagBitTracePrint (ctxt.pBitFldList, "");
      printf ("\n");
   }

   /* Verify buffer is byte-aligned */
   OSRTASSERT (ctxt.buffer.bitOffset == 8 || ctxt.buffer.bitOffset == 0);

   /* Write encoded message to output file */

   ret = rtxFileWriteBinary (filename, msgptr, len);
   if (0 != ret) {
      printf ("rtxFileWriteBinary failed; status = %d\n", ret);
      rtxErrPrint (&ctxt);
      return ret;
   }

   rtxFreeContext (&ctxt);

   return 0;
}
