#include "TS24501Msgs.h"
#include "rtxsrc/rtxMemLeakCheck.h"
#include "rtxsrc/rtxCharStr.h"
#include "rtxsrc/rtxDiag.h"
#include "rtxsrc/rtxDiagBitTrace.h"
#include "rtxsrc/rtxError.h"
#include "rtxsrc/rtxFile.h"
#include "rtxsrc/rtxPrint.h"
#include "rt3gppsrc/rt3gppNasSec.h"

#define MAXMSGLEN 300

int main (int argc, char** argv)
{
   TS24501Msg_PDU pdu;
   TS24501Msg_RemoteUEReport data;
   TS24501Msg_RemoteUEReport* pvalue = &data;
   OSCTXT	ctxt;
   OSOCTET	msgbuf[MAXMSGLEN], *msgptr;
   const char*  filename = "message.dat";
   OSSIZE       len;
   int		i, ret;
   OSBOOL verbose = FALSE;
   OSCrypt128Key integKey = {
      0x01, 0x23, 0x45, 0x67, 0x89, 0xab, 0xcd, 0xef,
      0x98, 0x76, 0x54, 0x32, 0x10, 0xfe, 0xdc, 0xba
   };
   OSCrypt128Key encryptKey = {
      0x98, 0x76, 0x54, 0x32, 0x10, 0xfe, 0xdc, 0xba,
      0x01, 0x23, 0x45, 0x67, 0x89, 0xab, 0xcd, 0xef
   };

   DECLARE_MEMLEAK_DETECTOR;

   /* Process command line arguments */

   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else {
            printf ("usage: writer [-v]\n");
            printf ("   -v  verbose mode: print trace info\n");
            return 0;
         }
      }
   }

   /* Initialize context structure */

   ret = rtInitContext (&ctxt);
   if (0 != ret) {
      printf ("rtInitContext failed; status = %d\n", ret);
      rtxErrPrint (&ctxt);
      return ret;
   }

   ret = rtx3gppSecInitialize(&ctxt);
   if (0 != ret) {
      rtxErrPrint (&ctxt);
   }

   ret = rtx3gppAssignAlgorithmKeys (&ctxt, &integKey, &encryptKey,
      OS3GPPSecAlgorithm_AES, OS3GPPSecAlgorithm_AES);
   if ( ret == RTERR_NOTSUPP ) {
      printf ("Warning: unable to set AES algorithm; falling back to "
              "NULL algorithm.\n");
      rtxErrReset (&ctxt);
      ret = rtx3gppAssignAlgorithmKeys (&ctxt, NULL, NULL, 
         OS3GPPSecAlgorithm_NULL_ALG, OS3GPPSecAlgorithm_NULL_ALG);
   }
   if (0 != ret) {
      rtxErrPrint (&ctxt);
   }

   ctxt.p3gppSec->count = 15;
   ctxt.p3gppSec->bearerId = 0;
   ctxt.p3gppSec->direction = 0; /*uplink*/

   /* Populate data structure */

   asn1Init_TS24501Msg_PDU (&pdu);
   pdu.m.secHdrPresent = 1;
   pdu.secHdr.protoDiscr = TS24501Msg_NAS5GProtoDiscr_mobMgmt5G;
   pdu.secHdr.secHdrType = TS24501Msg_NAS5GSecHdrType_integProt;
   pdu.secHdr.msgAuthCode = 40000;
   pdu.secHdr.seqNumber  = 80;
   pdu.protoDiscr = TS24501Msg_NAS5GProtoDiscr_sessMgmt5G;
   pdu.hdrData.u.sm.pduSessIdent = TS24501Msg_PDUSessionIdentity_pduId10;
   pdu.hdrData.u.sm.procTransIdent = 120;

   asn1SetTC_TS24501Msg_PDU_obj_RemUEReport5G (&ctxt, &pdu, pvalue);

   asn1Init_TS24501Msg_RemoteUEReport (pvalue);

   pvalue->m.connectedCtxtPresent = TRUE;
   pvalue->connectedCtxt.length = 0; /* length will be calculated */
   pvalue->connectedCtxt.numUECtxts = 1;
   pvalue->connectedCtxt.contexts.n = 1;
   pvalue->connectedCtxt.contexts.elem = rtxMemAllocArrayZ(&ctxt, 1,
                                            TS24501IE_RemoteUEContext);
   pvalue->connectedCtxt.contexts.elem[0].contextLength = 0; /* length will be calculated */
   pvalue->connectedCtxt.contexts.elem[0].ueidFormat = 1;
   pvalue->connectedCtxt.contexts.elem[0].ueidType = TS24501IE_UEIDTypes_cpPrukId;
   pvalue->connectedCtxt.contexts.elem[0].ueidLength = 8;
   {
      pvalue->connectedCtxt.contexts.elem[0].ueid.numocts = 8;
      static const OSOCTET tmpdata[] = { 0xfe, 0xed, 0x00, 0xff, 0xcd, 0xdc, 0x12, 0x34 };
      OSCRTLSAFEMEMCPY((void*)pvalue->connectedCtxt.contexts.elem[0].ueid.data, 8, tmpdata, 8);
   }
   pvalue->connectedCtxt.contexts.elem[0].m.spare2Present = TRUE;
   pvalue->connectedCtxt.contexts.elem[0].spare2 = 0;
   pvalue->connectedCtxt.contexts.elem[0].m.upri4iPresent = TRUE;
   pvalue->connectedCtxt.contexts.elem[0].upri4i = TRUE;
   pvalue->connectedCtxt.contexts.elem[0].m.tpri4iPresent = TRUE;
   pvalue->connectedCtxt.contexts.elem[0].tpri4i = FALSE;
   pvalue->connectedCtxt.contexts.elem[0].m.protocolPresent = TRUE;
   pvalue->connectedCtxt.contexts.elem[0].protocol = TS24501IE_RemoteUEProtocols_ipv4;
   {
      pvalue->connectedCtxt.contexts.elem[0].addrInfo.u.ipv4.numocts = 4;
      static const OSOCTET tmpdata[] = { 0xfe, 0xed, 0x00, 0xff };
      OSCRTLSAFEMEMCPY((void*)pvalue->connectedCtxt.contexts.elem[0].addrInfo.u.ipv4.data, 4, tmpdata, 4);
   }

   /* Enable bit tracing */

   if (verbose) {
      rtxDiagCtxtBitFieldListInit (&ctxt);
   }

   /* Encode data */

   rtxCtxtSetBufPtr (&ctxt, msgbuf, sizeof(msgbuf));

   ret = NASEnc_TS24501Msg_PDU (&ctxt, &pdu);
   if (0 != ret) {
      printf ("encode PDU failed; status = %d\n", ret);
      rtxErrPrint (&ctxt);
      printf ("Bit fields:\n");
      rtxDiagBitTracePrint (ctxt.pBitFldList, "");
      printf ("\n");
      rtxFreeContext (&ctxt);
      return ret;
   }
   msgptr = rtxCtxtGetMsgPtr (&ctxt);
   len = rtxCtxtGetMsgLen (&ctxt);

   if (verbose) {
      printf ("Encoding was successful\n");
      printf ("Hex dump of encoded record:\n");
      rtxHexDump (msgptr, len);
      printf ("\n");
      printf ("Bit fields:\n");
      rtxDiagBitTracePrint (ctxt.pBitFldList, "");
      printf ("\n");
   }

   /* Verify buffer is byte-aligned */
   OSRTASSERT (ctxt.buffer.bitOffset == 8 || ctxt.buffer.bitOffset == 0);

   /* Write encoded message to output file */

   ret = rtxFileWriteBinary (filename, msgptr, len);
   if (0 != ret) {
      printf ("rtxFileWriteBinary failed; status = %d\n", ret);
      rtxErrPrint (&ctxt);
      return ret;
   }

   rtxFreeContext (&ctxt);

   return 0;
}
